/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_GST_RENDERER_PLUGIN_H
#define MAFW_GST_RENDERER_PLUGIN_H

#include <QObject>
#include <MafwPlugin.h>

class MafwGstRendererPlugin : public QObject, public MafwPlugin
{
    Q_OBJECT
    Q_INTERFACES(MafwPlugin)

public:
    void initialize(MafwInternalRegistry* registry);
    ~MafwGstRendererPlugin();

    QString name() const;

protected:
    MafwInternalRegistry* m_registry;
    
private:
    QList<QString> m_rendererIds;

    /**
    * Loads gst-renderers from config file.
    * Config file contais "renderers" and "in-process-renderers" arrays.
    * Renderer Array contains gst-renderer's id and it's "friendly" name.
    * Syntax of the "renderers" array:
    *    [renderers]
    *    1\Id=mafw_gst_renderer
    *    1\FriendlyName=MafwGstRenderer
    *    2\Id=mafw_gst_internal_video_renderer
    *    2\FriendlyName=MafwGstVideoRenderer
    *    size=2
    * If config file is deleted of empty we create default renderer with "mafw_gst_renderer" uuid.
    **/
    void loadRenderers(const QString& rendererArrayKey);

};

#endif
