/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#include "MafwBlankingPreventer.h"
#include <qmdisplaystate.h>
#include <QDebug>

/* Interval of recalling setBlankingPause in seconds */
const int BLANKING_TIMER_INTERVAL=45;

MafwBlankingPreventer::MafwBlankingPreventer(QObject* parent) : QObject(parent)
{
    connect( &m_refreshTimer, SIGNAL(timeout()), this, SLOT(refresh()));
    m_refreshTimer.setInterval( BLANKING_TIMER_INTERVAL*1000 );
    m_display = new MeeGo::QmDisplayState(this);
}

void MafwBlankingPreventer::blankingProhibit()
{
    qDebug() << "MafwBlankingPreventer::blankingProhibit";
    refresh();
    m_refreshTimer.start();
}

void MafwBlankingPreventer::blankingAllow()
{
    qDebug() << "MafwBlankingPreventer::blankingAllow";
    m_refreshTimer.stop();
    m_display->cancelBlankingPause();
}

void MafwBlankingPreventer::refresh()
{
    bool success = m_display->setBlankingPause();
    qDebug() << "MafwBlankingPreventer::refresh success" << success;
}

